#library "golf"
#include "zcommon.acs"

global int 56:camheight;

#include "golf_funcs.acs"
#include "golf_char.acs"
#include "golf_messages.acs"

#define MAX_PLAYERS 64

// Hudmessages:
// 10 - Death countdown
// 11 - YAY EVERYBODY GOT IN
// 12 - Cameramode on
// 13 - "Game ends in..."
// 14 - "X minutes"
// 15 - Congratulations 1
// 16 - Congratulations 2
// 17 - Paint select
// 18 - Get a new ball?
// 19 - Y, new ball, n, keep playing
// 20 - x/y/z debug info for ball positioning
// 21 - Message for the player exceeding 5x the par
// 22-23 - Hellshot activator thingy
// 30-94 - Player info on HUD (I swear to god this will be awful if 64 players actually play)

// TIDs:
// 100+ - Player numbers
// 1500+ - Cameras // Previously 1000+, but turns out 8BDM uses that for the players.
// 2000+ - Balls and colors for ball indicators
// 2500+ - Landed ball

int ballx[MAX_PLAYERS], bally[MAX_PLAYERS], ballz[MAX_PLAYERS], balltid[MAX_PLAYERS];
int startballx[MAX_PLAYERS], startbally[MAX_PLAYERS], startballz[MAX_PLAYERS];
int PlayersInGame, PlayersFinished, PlayersFinishedScoreboard;
int PlayerStrokes[MAX_PLAYERS];
int PlayerFinishedNames[MAX_PLAYERS], PlayerFinishedStrokes[MAX_PLAYERS], IFinished[MAX_PLAYERS];
int PlayerDeliberating[MAX_PLAYERS];

script "GOLF_UNLOADING" UNLOADING
{
    // Take EVERYTHING for a fresh restart on the next map
    TakeInventory("MaySwingAgain",1);
    TakeInventory("BetweenShots",1);
    TakeInventory("SwingStage1",1);
    TakeInventory("CancelSwing",1);
    TakeInventory("CameraOn",1);
    TakeInventory("FinishedHole",1);
    TakeInventory("SelectingPaint",1);
    TakeInventory("KillTheBall",1);
    TakeInventory("NotPlayingGolf",1);
    TakeInventory("ClubHit",1);
    TakeInventory("ClubSwing",1);
    TakeInventory("ClubSwinging",1);

    TakeInventory("CourseNumberHack",0x7FFFFFFF);
    TakeInventory("CourseParHack",0x7FFFFFFF);
    TakeInventory("CourseAceHack",0x7FFFFFFF);
    TakeInventory("ColorType",0x7FFFFFFF);
    TakeInventory("StrokeCounter",0x7FFFFFFF);
    TakeInventory("SwingArc",0x7FFFFFFF);
}

script "GOLF_DEATH" DEATH
{
    GiveInventory("MaySwingAgain",1);
    GiveInventory("BetweenShots",1);
    TakeInventory("SwingStage1",1);
    TakeInventory("PoweredUp",1);
    TakeInventory("CameraOn",1);
    TakeInventory("SelectingPaint",1);
    TakeInventory("KillTheBall",1);
    TakeInventory("ClubHit",1);
    TakeInventory("ClubSwing",1);
    TakeInventory("ClubSwinging",1);
}

script "GOLF_OPEN" OPEN
{
    int TimeSet, TimeTotal;
    int TimeMins, TimeSecs, TimeCount, TimeCounted;

    PlayersInGame = 0;
    PlayersFinished = 0;
    PlayersFinishedScoreboard = 0;

    int s = unusedTID(37000, 47000);

    // Pretty sure GZDoom won't be getting the Doomsphere for a while, but in
    // case GZDoom ever complains about ConsoleCommand, this is why.
    if (SpawnForced("Doomsphere",0,0,0,s,0))
    {
        Thing_Remove(s);
        if (GetCvar("golf_debug") == 1) { Log(s:"Zandronum check successful."); }
        ConsoleCommand("compat_clientssendfullbuttoninfo 1");
        ConsoleCommand("compat_limited_airmovement 1");
        ConsoleCommand("sv_disallowsuicide 0");
        ConsoleCommand("sv_nocrouch 1");
        ConsoleCommand("sv_chasecam 1");
    }
    else
    {
        if (GetCvar("golf_debug") == 1) { Log(s:"Zandronum check failed, assuming ZDoom."); }
    }
    SetAirControl(0.66);

    delay(1);

    while(1)
    {
        if (PlayersFinished >= PlayersInGame && PlayersInGame > 0 && GameType() != GAME_SINGLE_PLAYER)
        // Previously there was a check for PlayersInGame > 1 here, but what if
        // someone's idling in a server? They should be able to complete a map.
        {
            if (GetCvar("golf_debug") == 1) { Log(s:"All players filled the hole."); }
            AmbientSound("hole/finished",127);
            SetHudSize(320,200,0);
            SetFont("BIGFONT");
            HudMessageBold(s:"COMPLETED HOLE!";HUDMSG_FADEOUT,11,CR_GOLD,160.0,25.0,2.0,1.0);
            delay(35*5);
            Exit_Normal(0);
            delay(1);
            terminate;
        }

        if (PlayersFinished > 0 && PlayersInGame > 1 && GetCvar("golf_endgame") == 1 && GetCvar("golf_endmins") > 0 && GameType() != GAME_SINGLE_PLAYER)
        // Isn't this last check redundant, with PlayersInGame > 1 being checked?
        // Ah well, might as well. Just In Case, and all that.
        {
            if (TimeSet == 0)
            {
                if (GetCvar("golf_debug") == 1) { Log(s:"Setting timer value."); }
                TimeSet = 1;
                TimeMins = GetCvar("golf_endmins");
                TimeTotal = TimeMins;
                if (TimeCounted > 0)
                {
                    if (GetCvar("golf_debug") == 1) { Log(s:"Subtracting timer value."); }
                    TimeMins -= TimeCounted;
                    if (TimeMins < 0)
                    {
                       if (GetCvar("golf_debug") == 1) { Log(s:"Timer value was subtracted to less than zero."); }
                       Exit_Normal(0);
                    }
                }
            }
            if (TimeTotal != GetCvar("golf_endmins"));
            {
                if (GetCvar("golf_debug") == 1) { Log(s:"Timer was changed, resetting value."); }
                TimeSet = 0;
            }
            SetHudSize(320,200,0);
            SetFont("SMALLFONT");
            HudMessage(s:"Game ends in...";HUDMSG_PLAIN,13,CR_GOLD,160.0,10.0,0.2);
            if (TimeSecs >= 10) { HudMessage(i:TimeMins,s:":",i:TimeSecs;HUDMSG_PLAIN,14,CR_WHITE,160.0,18.0,0.2); }
            else { HudMessage(i:TimeMins,s:":0",i:TimeSecs;HUDMSG_PLAIN,14,CR_WHITE,160.0,18.0,0.2); }
            timecount++;
            if (TimeCount >= 35)
            {
                TimeSecs--;
                if (TimeSecs < 0)
                {
                   TimeCounted++;
                   if (TimeMins > 0)
                   {
                       if (GetCvar("golf_debug") == 1) { Log(s:"Timer reducing a minute."); }
                       TimeMins--;
                       TimeSecs = 59;
                   }
                   else
                   {
                       if (GetCvar("golf_debug") == 1) { Log(s:"Timer expired."); }
                       Exit_Normal(0);
                   }
                }
                TimeCount = 0;
            }
        }
        else
        {
            TimeSet = 0;
            TimeCount = 0;
            TimeCounted = 0;
            TimeSecs = 0;
        }

        delay(1);
    }
}

script "GOLF_SCOREBOARD" OPEN
{
    int x;

    delay(1);

    while(1)
    {
        SetFont("SMALLFONT");
        SetHudSize(640,400,0);
        /*for (x = 0; x < PlayersInGame; x++)//MAX_PLAYERS; ++x)
        {
            //if (CheckActorInventory(100+x,"ImAlive") == 1)
            //{
                HudMessageBold(s:"- ",n:x+1,s:" \cj:\cg ",d:PlayerStrokes[x]; HUDMSG_PLAIN, 30+x, CR_UNTRANSLATED, 20.0, 5.0+(x*10.0), 0.2);
            //}
        }
        if (x > MAX_PLAYERS || x > PlayersInGame) { x = 0; }*/
        if (PlayersFinishedScoreboard > 0)
        {
            for (x = 0; x < PlayersFinishedScoreboard; x++)//MAX_PLAYERS; ++x)
            {
                HudMessageBold(s:"\cj",s:PlayerFinishedNames[x],s:" \cf:\cj ",d:PlayerFinishedStrokes[x]; HUDMSG_PLAIN, 30+x, CR_UNTRANSLATED, 80.0, 20.0+(x*10.0), 0.2);
            }
        }
        delay(1);
    }
}

script "GOLF_CAMERA" ENTER CLIENTSIDE
{
    int CameraTurnedOn, CameraSpeed, CameraMoving, cameratid;
    int buttons, oldbuttons;
    int forwardmove, sidemove;
    int a, a2, p, p2;
    int x, y, z;

    int pln;
    int pitchChange, yawChange;
    int i, j;
    int pitchSlope, pitchPeak;
    int yawSlope, yawPeak;
    int curpitch, lastpitch;
    int curyaw, lastyaw;

    delay(1); // For the camera to initially spawn for the player.

    SetHudSize(320,200,0);
    a = GetActorAngle(0);
    a2 = GetActorAngle(0);
    p = GetActorPitch(0);
    p2 = GetActorPitch(0);
    pitchchange = 0;
    yawchange = 0;
    pln = PlayerNumber();
    cameratid = 1500+pln;

    while(1)
    {
        buttons = GetPlayerInput(-1, INPUT_BUTTONS);
        oldbuttons = GetPlayerInput(-1, INPUT_OLDBUTTONS);
        forwardmove = GetPlayerInput(-1, INPUT_FORWARDMOVE);
        sidemove = GetPlayerInput(-1, INPUT_SIDEMOVE);
        a = GetActorAngle(0);
        p = GetActorPitch(0);
        a2 = GetActorAngle(cameratid);
        p2 = GetActorPitch(cameratid);
        x = GetActorX(0);
        y = GetActorY(0);
        z = GetActorZ(0);

        if (buttons & BT_SPEED) { CameraSpeed = 40; }
        else { CameraSpeed = 20; }
        
        if (CheckInventory("CameraOn") == 1)
        {
            //SetFont("BIGFONT");
            //HudMessage(s:"CAMERA VISION";HUDMSG_PLAIN,12,CR_GOLD,160.0,15.0,0.2);
            SetFont("CAMVISN1");
            HudMessage(s:"a";HUDMSG_PLAIN,12,CR_UNTRANSLATED,160.0,100.0,0.1);

            if (sidemove > 0)
            {
                CameraMoving = 1;
                SetActorVelocity(cameratid, CameraSpeed*cos(a2-0.25), CameraSpeed*sin(a2-0.25), 0, 0, 0);
            }
            if (sidemove < 0)
            {
                CameraMoving = 1;
                SetActorVelocity(cameratid, CameraSpeed*cos(a2+0.25), CameraSpeed*sin(a2+0.25), 0, 0, 0);
            }
            if (forwardmove > 0)
            {
                CameraMoving = 1;
                if (sidemove > 0) { SetActorVelocity(cameratid, CameraSpeed*cos(a2-0.125), CameraSpeed*sin(a2-0.125), -(CameraSpeed)*sin(p2), 0, 0); }
                else if (sidemove < 0) { SetActorVelocity(cameratid, CameraSpeed*cos(a2+0.125), CameraSpeed*sin(a2+0.125), -(CameraSpeed)*sin(p2), 0, 0); }
                else { SetActorVelocity(cameratid, CameraSpeed*cos(a2), CameraSpeed*sin(a2), -(CameraSpeed)*sin(p2), 0, 0); }
            }
            if (forwardmove < 0)
            {
                CameraMoving = 1;
                if (sidemove > 0) { SetActorVelocity(cameratid, -CameraSpeed*cos(a2+0.125), -CameraSpeed*sin(a2+0.125), (CameraSpeed)*sin(p2), 0, 0); }
                else if (sidemove < 0) { SetActorVelocity(cameratid, -CameraSpeed*cos(a2-0.125), -CameraSpeed*sin(a2-0.125), (CameraSpeed)*sin(p2), 0, 0); }
                else { SetActorVelocity(cameratid, -CameraSpeed*cos(a2), -CameraSpeed*sin(a2), (CameraSpeed)*sin(p2), 0, 0); }
            }

            pitchChange += GetPlayerInput(pln, INPUT_PITCH) * 256; // ANGMULT was previously here, as the 256.
            yawChange += GetPlayerInput(pln, INPUT_YAW) * 256; // Same.

            curpitch = pitchPeak;
            curyaw = yawPeak;

            i = GetActorAngle(cameratid);
            j = GetActorPitch(cameratid);

            SetActorAngle(cameratid, i + ((curyaw - lastyaw) / 256)); // As above.
            SetActorPitch(cameratid, j - ((curpitch - lastpitch) / 256)); // As above.

            if (GetActorPitch(cameratid) > 0.25) { SetActorPitch(cameratid,0.25); }
            if (GetActorPitch(cameratid) < -0.25) { SetActorPitch(cameratid,-0.25); }

            pitchPeak = pitchChange;
            yawPeak = yawChange;
            pitchSlope = quadSlope(0, 0, 1, pitchChange, 1); // UpdateTics was previously in the third int, defined as 1.
            yawSlope = quadSlope(0, 0, 1, yawChange, 1); // Same.

            pitchChange = 0;
            yawChange = 0;

            lastpitch = 0;
            lastyaw   = 0;

            if (CameraMoving == 0) { SetActorVelocity(cameratid,0,0,0,0,0); }
            CameraMoving = 0;
        }
        else
        {
            SetActorPosition(cameratid,x,y,z+64.0,0);
            SetActorAngle(cameratid,a);
            SetActorPitch(cameratid,p);
        }

        if (CheckInventory("CameraOn") == 0)
        {
            if (CameraTurnedOn == 1)
            {
                LocalAmbientSound("camera/off",127);
                ChangeCamera(0,0,0);
                CameraTurnedOn = 0;
            }
        }
        else
        {
            if (CameraTurnedOn == 0)
            {
                LocalAmbientSound("camera/on",127);
                ChangeCamera(cameratid,0,0);
                CameraTurnedOn = 1;
            }
        }

        delay(1);
    }
}

script "GOLF_ENTER" ENTER
{
    int buttons, oldbuttons;
    int pln, cameratid;
    int a, x, y, z, ang, pit;
    int StrokeCount;
    int deathcountdown;

    //if (ConsolePlayerNumber() != PlayerNumber()) { terminate; }

    pln = PlayerNumber();
    cameratid = 1500+pln;
    PlayerDeliberating[pln] = 0;

    ballx[pln] = GetActorX(0); // Set the starting positions.
    bally[pln] = GetActorY(0);
    ballz[pln] = GetActorZ(0);
    startballx[pln] = GetActorX(0); // For in case the players need to reset the ball...
    startbally[pln] = GetActorY(0);
    startballz[pln] = GetActorZ(0);

    SetActorVelocity(0,0,0,0,0,0);
    SetPlayerProperty(0,0,PROP_TOTALLYFROZEN);
    SetPlayerProperty(0,0,PROP_FROZEN);

    if (GameType() == GAME_NET_DEATHMATCH) { SetActorProperty(0,APROP_SPECIES,"DMPlayer"); }
    else { GiveInventory("DeathsplosionResistance",1); }
        
    if (!ThingCountName("ChaseCam2",cameratid))
    {
        SpawnForced("ChaseCam2",x,y,z+64.0,cameratid, a >> 8);
    }

    // Just to bloody make sure.
    GiveInventory("CancelSwing",1);
    GiveInventory("ImAlive",1);
    GiveInventory("MaySwingAgain",1);
    GiveInventory("BetweenShots",1);
    GiveInventory("NoClipDeactivate",1);
    TakeInventory("NotPlayingGolf",1);
    TakeInventory("SwingStage1",1);
    TakeInventory("KillTheBall",1);
    TakeInventory("CancelSwing",1);
    TakeInventory("CameraOn",1);
    TakeInventory("FinishedHole",1);
    TakeInventory("SelectingPaint",1);
    TakeInventory("ClubHit",1);
    TakeInventory("ClubSwing",1);
    TakeInventory("ClubSwinging",1);

    TakeInventory("ColorType",0x7FFFFFFF);
    TakeInventory("StrokeCounter",0x7FFFFFFF);
    TakeInventory("SwingArc",0x7FFFFFFF);

    if (CheckInventory("LightClub") == 0) { GiveInventory("LightClub",1); }
    if (CheckInventory("MediumClub") == 0) { GiveInventory("MediumClub",1); }
    if (CheckInventory("HeavyClub") == 0) { GiveInventory("HeavyClub",1); }
    if (CheckInventory("Painter") == 0 && CheckInventory("IsLordBlaz") == 0)
    {
        if (GetCvar("golf_nopaint") == 0) { GiveInventory("Painter",1); }
    }
    if (CheckInventory("PaintWand") == 0 && CheckInventory("IsLordBlaz") == 1)
    {
        if (GetCvar("golf_nopaint") == 0) { GiveInventory("PaintWand",1); }
    }

    SetActorProperty(0,APROP_ACCURACY,0);

    //if (GetLevelInfo(LEVELINFO_LEVELNUM) < 100) { GiveInventory("NotPlayingGolf",1); }
    if (ThingCountName("EndFlag",0) == 0) { GiveInventory("NotPlayingGolf",1); }

    if (CheckInventory("IsDoomguy") == 1) { SetActorProperty(0,APROP_ACCURACY,8); }
    if (CheckInventory("IsMaestro") == 1) { SetActorProperty(0,APROP_ACCURACY,2); }
    if (CheckInventory("IsCygnis") == 1) { SetActorProperty(0,APROP_ACCURACY,16); }
    if (CheckInventory("IsCKid") == 1) { SetActorProperty(0,APROP_ACCURACY,0); }
    if (CheckInventory("IsRekkr") == 1) { SetActorProperty(0,APROP_ACCURACY,8); }
    if (CheckInventory("IsThi") == 1) { SetActorProperty(0,APROP_ACCURACY,6); }
    if (CheckInventory("IsLordBlaz") == 1) { SetActorProperty(0,APROP_ACCURACY,0); }
    if (CheckInventory("IsRenegade") == 1) { SetActorProperty(0,APROP_ACCURACY,8); }
    if (CheckInventory("IsDaina") == 1) { SetActorProperty(0,APROP_ACCURACY,6); }
    if (CheckInventory("IsVesper") == 1) { SetActorProperty(0,APROP_ACCURACY,8); }
    if (CheckInventory("IsCorzo") == 1) { SetActorProperty(0,APROP_ACCURACY,8); }
    if (CheckInventory("IsSquare") == 1) { SetActorProperty(0,APROP_ACCURACY,4); }
    if (CheckInventory("IsMsSkeltal") == 1) { SetActorProperty(0,APROP_ACCURACY,8); }

    PlayersInGame++;

    while(1)
    {
        a = GetActorAngle(0);
        buttons = GetPlayerInput(-1, INPUT_BUTTONS);
        oldbuttons = GetPlayerInput(-1, INPUT_OLDBUTTONS);
        StrokeCount = CheckInventory("StrokeCounter");
        x = GetActorX(0);
        y = GetActorY(0);
        z = GetActorZ(0);
        if (CheckInventory("CameraOn") == 0)
        {
            ang = GetActorAngle(0);
            pit = GetActorPitch(0);
        }

        SetHudSize(320,200,0); // For all HudMessage stuff.
        PlayerStrokes[pln] = StrokeCount;

        // Just in case of ClearInventory.
        if (CheckInventory("MegaBuster") == 1) { TakeInventory("MegaBuster",1); }
        if (CheckInventory("Pistol") == 1) { TakeInventory("Pistol",1); }
        if (CheckInventory("Fist") == 1) { TakeInventory("Fist",1); }

        if (CheckInventory("SwingStage1") == 1) { SetActorProperty(0,APROP_JUMPZ,0); } // brute force
        else { SetActorProperty(0,APROP_JUMPZ,12.0); } // brute fucking force

        if (CheckInventory("SwingStage1") == 1 && CheckInventory("NotPlayingGolf") == 0)
            { SetActorProperty(0,APROP_MASS,0x7FFFFFFF); }
        else
        {
            if (CheckInventory("IsDoomguy") == 1) { SetActorProperty(0,APROP_MASS,100); }
            if (CheckInventory("IsMaestro") == 1) { SetActorProperty(0,APROP_MASS,1000); }
            if (CheckInventory("IsCygnis") == 1) { SetActorProperty(0,APROP_MASS,100); }
            if (CheckInventory("IsCKid") == 1) { SetActorProperty(0,APROP_MASS,64); }
            if (CheckInventory("IsRekkr") == 1) { SetActorProperty(0,APROP_MASS,100); }
            if (CheckInventory("IsThi") == 1) { SetActorProperty(0,APROP_MASS,100); }
            if (CheckInventory("IsLordBlaz") == 1) { SetActorProperty(0,APROP_MASS,80); }
            if (CheckInventory("IsRenegade") == 1) { SetActorProperty(0,APROP_MASS,100); }
            if (CheckInventory("IsDaina") == 1) { SetActorProperty(0,APROP_MASS,150); } // Heh.
            if (CheckInventory("IsVesper") == 1) { SetActorProperty(0,APROP_MASS,100); }
            if (CheckInventory("IsCorzo") == 1) { SetActorProperty(0,APROP_MASS,100); }
            if (CheckInventory("IsSquare") == 1) { SetActorProperty(0,APROP_MASS,100); }
            if (CheckInventory("IsMsSkeltal") == 1) { SetActorProperty(0,APROP_MASS,100); }
        }

        if (GetCvar("golf_survival") == 1 && CheckInventory("SwingStage1") == 1)
        {
            if (CheckInventory("TimeToBom") == 0)
            {
                GiveInventory("TimeToBom",1);
                LocalAmbientSound("bomb/start",127);
            }
        }

        //if (CheckInventory("PoweredUp") == 1) { GiveInventory("HellshotFireSpawner",1); }

        if (CheckInventory("TimeToBom") == 1)
        {
            deathcountdown++;
            SetFont("BIGFONT");
            if (deathcountdown == 35)
            {
                LocalAmbientSound("bomb/count",127);
                GiveInventory("Countdown5Trigger",1);
                HudMessage(s:"5";HUDMSG_FADEOUT,10,CR_GOLD,160.0,25.0,0.5,0.25);
            }
            if (deathcountdown == 70)
            {
                LocalAmbientSound("bomb/count",127);
                GiveInventory("Countdown4Trigger",1);
                HudMessage(s:"4";HUDMSG_FADEOUT,10,CR_GOLD,160.0,25.0,0.5,0.25);
            }
            if (deathcountdown == 105)
            {
                LocalAmbientSound("bomb/count",127);
                GiveInventory("Countdown3Trigger",1);
                HudMessage(s:"3";HUDMSG_FADEOUT,10,CR_GOLD,160.0,25.0,0.5,0.25);
            }
            if (deathcountdown == 140)
            {
                LocalAmbientSound("bomb/count",127);
                GiveInventory("Countdown2Trigger",1);
                HudMessage(s:"2";HUDMSG_FADEOUT,10,CR_GOLD,160.0,25.0,0.5,0.25);
            }
            if (deathcountdown == 175)
            {
                LocalAmbientSound("bomb/count",127);
                GiveInventory("Countdown1Trigger",1);
                HudMessage(s:"1";HUDMSG_FADEOUT,10,CR_GOLD,160.0,25.0,0.5,0.25);
            }
            if (deathcountdown == 210)
            {
                LocalAmbientSound("bomb/going",127);
                SpawnForced("DeathSplode",x,y,z+24,0,0);
                DamageThing(0,0);
                TakeInventory("TimeToBom",1);

                PlayerStrokes[pln] = 0;
                TakeInventory("StrokeCounter",0x7FFFFFFF);
                ballx[pln] = startballx[pln];
                bally[pln] = startbally[pln];
                ballz[pln] = startballz[pln];

                Thing_Remove(2000+PlayerNumber());
                Thing_Remove(2500+PlayerNumber());
            }
        }
        else
        {
            deathcountdown = 0;
        }

        if (buttons & BT_ZOOM && !(oldbuttons & BT_ZOOM) && CheckInventory("SelectingPaint") == 0 && GetActorProperty(0,APROP_HEALTH) > 0 && GetCvar("golf_nocam") == 0 && PlayerDeliberating[pln] == 0)
        {
            if (CheckInventory("CameraOn") == 0)
            {
                //LocalAmbientSound("camera/on",127);
                SetPlayerProperty(0,1,PROP_TOTALLYFROZEN);
                //ChangeCamera(cameratid,0,0);
                GiveInventory("CameraOn",1);
            }
            else
            {
                //if (CheckInventory("BetweenShots") == 1) { SetPlayerProperty(0,0,PROP_FROZEN); }
                //LocalAmbientSound("camera/off",127);
                SetPlayerProperty(0,0,PROP_TOTALLYFROZEN); 
                //ChangeCamera(0,0,0);
                TakeInventory("CameraOn",1);
            }
        }

        /*if ((CheckInventory("HellshotAmmo") >= 1750 && CheckInventory("HellshotStock") == 0) || (CheckInventory("HellshotAmmo") >= 3500 && CheckInventory("HellshotStock") == 1))
        {
            FadeRange(190,96,0,0.666,0,0,0,0,0.75);
            LocalAmbientSound("power/get",127);
            GiveInventory("HellshotStock",1);
        }*/

        if (GetCvar("golf_nocam") == 1 && CheckInventory("CameraOn") == 1)
        {
            SetPlayerProperty(0,0,PROP_TOTALLYFROZEN);
            TakeInventory("CameraOn",1);
        }

        // brute force brute force brute force brute force brute force brute force brute force brute force brute force brute force brute force brute force brute force
        if (CheckInventory("SwingStage1") > 0)
        {
            TakeInventory("Painter",1);
            TakeInventory("PaintWand",1);
            //TakeInventory("Leela",1);
        }
        else
        {
            if (GetCvar("golf_nopaint") == 0)
            {
                if (CheckInventory("IsLordBlaz") == 0) { GiveInventory("Painter",1); }
                if (CheckInventory("IsLordBlaz") == 1) { GiveInventory("PaintWand",1); }
            }
            else { TakeInventory("Painter",1); TakeInventory("PaintWand",1); }

            //if (CheckInventory("IsRenegade") == 1) { GiveInventory("Leela",1); }
        }

        if (getCvar("golf_debug") == 1)
        {
            SetFont("SMALLFONT");
            HudMessage(f:ballx[pln],s:"\n",f:bally[pln],s:"\n",f:ballz[pln];HUDMSG_PLAIN,20,CR_WHITE,280.0,20.0,0.2);
        }

        delay(1);
    }
}

script "GOLF_DECORATE" (int fuck)
{
    int x, y, z;
    int fuckthis, dickthis, thegallofthisshit, quoteshenanigans, fuckthisshit;
    int pln;
    int buttons, oldbuttons;
    int prevpaint, changepaint;

    switch (fuck)
    {
    case 0:
        x = GetActorX(0);
        y = GetActorY(0);
        z = GetActorZ(0);
        //PrintBold(i:GetActorZ(0) - GetActorFloorZ(0));
        //SetActivatorToTarget(0);
        if (CheckActorFloorTexture(0,"HOLFINIS"))
        {
            //SetActivator(0,AAPTR_MASTER);
            SetActivatorToTarget(0);
            if (CheckInventory("NotPlayingGolf") == 1) { terminate; }

            AmbientSound("hole/in",127);
            if (CheckInventory("StrokeCounter") <= CheckInventory("CourseAceHack")) { LocalAmbientSound("audience/roar",127); }
            else if (CheckInventory("StrokeCounter") <= CheckInventory("CourseParHack")) { LocalAmbientSound("audience/clap",127); }

            SetHudSize(320,200,0);
            SetFont("SMALLFONT");
            HudMessageBold(n:0,s:" finished";HUDMSG_FADEOUT,15,CR_GOLD,160.0,85.0,1.5,1.0);
            if (CheckInventory("StrokeCounter") > 1)
            {
                HudMessageBold(s:"in ",i:CheckInventory("StrokeCounter"),s:" strokes!";HUDMSG_FADEOUT,16,CR_GOLD,160.0,95.0,1.5,1.0);
                HudMessageBold(n:0,s:" finished in ",i:CheckInventory("StrokeCounter"),s:" strokes!";HUDMSG_LOG,0,CR_UNTRANSLATED,0.0,0.0,0.1);
            }
            else
            {
                HudMessageBold(s:"in ",i:CheckInventory("StrokeCounter"),s:" stroke!";HUDMSG_FADEOUT,16,CR_GOLD,160.0,95.0,1.5,1.0);
                HudMessageBold(n:0,s:" finished in ",i:CheckInventory("StrokeCounter"),s:" stroke!";HUDMSG_LOG,0,CR_UNTRANSLATED,0.0,0.0,0.1);
            }

            if (CheckInventory("StrokeCounter") > (CheckInventory("CourseParHack") + 15) && CheckInventory("CourseParHack") != 0)
            {
                quoteshenanigans = random(0,QUOTEAMOUNT - 1);
                HudMessageBold(s:EndingQuotes[quoteshenanigans];HUDMSG_FADEOUT,0,CR_WHITE,160.0,125.0,3.5,1.0);
            }

            GiveInventory("FinishedHole",1);
            GiveInventory("MaySwingAgain",1);
            GiveInventory("BetweenShots",1);
            IFinished[PlayerNumber()] = 1;
            PlayerFinishedNames[PlayersFinished] = Strparam(n:0);
            PlayerFinishedStrokes[PlayersFinished] = CheckInventory("StrokeCounter");
            PlayersFinished++;
            PlayersFinishedScoreboard++;
            if (GameType() == GAME_SINGLE_PLAYER || GetCvar("golf_endgame") == 2)
            {
                delay(35*5);
                Exit_Normal(0);
            }
        }
        else
        {
            if (GetOutOfBounds() == 1)//(CheckActorFloorTexture(0,"OUTBOUND"))
            {
                GiveInventory("KillMe",1); // Just In Case
                SetActivatorToTarget(0);
                if (CheckInventory("NotPlayingGolf") == 1) { terminate; }
                LocalAmbientSound("ball/badland",127);
                SetHudSize(320,200,0);
                SetFont("SMALLFONT");
                HudMessage(s:"Out of bounds!";HUDMSG_FADEOUT,15,CR_GOLD,160.0,85.0,1.5,1.0);
                HudMessage(s:"Alt-fire to try again.";HUDMSG_FADEOUT,16,CR_GOLD,160.0,95.0,1.5,1.0);
                //GiveInventory("KillTheBall",1);
                GiveInventory("MaySwingAgain",1);
                GiveInventory("BetweenShots",1);
                Thing_Remove(2000+PlayerNumber());
                //ACS_NamedExecuteAlways("GOLF_DECORATE",0,8,0,0);
            }
            else
            {
                SetActivatorToTarget(0);
                if (CheckInventory("NotPlayingGolf") == 1) { terminate; }
                ballx[PlayerNumber()] = x;
                bally[PlayerNumber()] = y;
                ballz[PlayerNumber()] = z;
                //SetActorPosition(0,x,y,z,0);
                LocalAmbientSound("ball/goodland",127);
                SetHudSize(320,200,0);
                SetFont("SMALLFONT");
                HudMessage(s:"Ball landed!";HUDMSG_FADEOUT,15,CR_GOLD,160.0,85.0,1.5,1.0);
                HudMessage(s:"Alt-fire to swing.";HUDMSG_FADEOUT,16,CR_GOLD,160.0,95.0,1.5,1.0);
                GiveInventory("KillTheBall",1);
                //GiveInventory("MaySwingAgain",1);
                GiveInventory("BetweenShots",1);
                ACS_NamedExecuteAlways("GOLF_DECORATE",0,8,0,0);
            }
        }
        break;

    case 1:
        SetPlayerProperty(0,0,PROP_FROZEN);
        if (CheckInventory("NotPlayingGolf") == 1)
        {
            GiveInventory("MaySwingAgain",1);
            GiveInventory("BetweenShots",1);
        }
        break;

    case 2:
        pln = PlayerNumber();
        if (CheckInventory("NotPlayingGolf") == 0)
        {
            LocalAmbientSound("return/teleport",127);
            FadeRange(0,255,0,0.5,0,0,0,0,1.0);
        }
        else { LocalAmbientSound("return/coopready",127); }
        SetActorVelocity(0,0,0,0,0,0);
        //SetActorPosition(0,ballx[PlayerNumber()],bally[PlayerNumber()],ballz[PlayerNumber()],0);
        if (CheckInventory("NotPlayingGolf") == 0)
        {
            // Remnants of terrible, terrible crimes go here.
            /*if (GameType() != GAME_NET_DEATHMATCH)
            {
                if (Spawn("NoClipCheck",ballx[pln],bally[pln],ballz[pln],9999+pln,0)) {}
                else
                {
                    //if (Warp(9999+pln,ballx[pln],bally[pln],ballz[pln],9999+pln)
                    //PrintBold(s:"don't fuck me");
                    //GiveInventory("NoClipActivate",1);
                }
            }*/
            Warp(0,ballx[pln],bally[pln],ballz[pln],0,WARPF_USECALLERANGLE | WARPF_ABSOLUTEPOSITION | WARPF_NOCHECKPOSITION | WARPF_STOP);
        }
        SetPlayerProperty(0,1,PROP_FROZEN);
        TakeInventory("BetweenShots",1);
        break;

    case 3:
        LocalAmbientSound("return/unlock",127);
        SetPlayerProperty(0,0,PROP_FROZEN);
        GiveInventory("BetweenShots",1);
        TakeInventory("SwingStage1",1); // Just in case.
        break;

    case 4:
        // REWRITE FIVE FUCKING GOOOOOOOO
        // oh my god why won't this fucking work
        //PrintBold(s:"fuck");
        if (GetOutOfBounds() == 1)//(CheckActorFloorTexture(0,"OUTBOUND"))
        {
            GiveInventory("KillMe",1); // Just In Case
            SetActivatorToTarget(0);
            if (CheckInventory("NotPlayingGolf") == 1) { terminate; }
            LocalAmbientSound("ball/badland",127);
            SetHudSize(320,200,0);
            SetFont("SMALLFONT");
            HudMessage(s:"Out of bounds!";HUDMSG_FADEOUT,15,CR_GOLD,160.0,85.0,1.5,1.0);
            HudMessage(s:"Alt-fire to try again.";HUDMSG_FADEOUT,16,CR_GOLD,160.0,95.0,1.5,1.0);
            //GiveInventory("KillTheBall",1);
            GiveInventory("MaySwingAgain",1);
            GiveInventory("BetweenShots",1);
            pln = PlayerNumber();
            Thing_Remove(2000+pln);
            //ACS_NamedExecuteAlways("GOLF_DECORATE",0,8,0,0);
        }
        else if (GetOutOfBounds() == 2)
        {
            ActivatorSound("ball/sand",127);
            //dickthis = GetActorProperty(0,APROP_GRAVITY);
            //SetActorProperty(0,APROP_GRAVITY,dickthis + 0.3);
            SetActorVelocity(0,0,0,0,0,0);
            SetActorState(0,"Death");
        }
        break;

    case 5:
        prevpaint = CheckInventory("ColorType");

        SetPlayerProperty(0,1,PROP_FROZEN);
        while (CheckInventory("SelectingPaint") == 1)
        {
            buttons = GetPlayerInput(-1, INPUT_BUTTONS);
            oldbuttons = GetPlayerInput(-1, INPUT_OLDBUTTONS);

            SetHudSize(320,200,0);

            if (buttons & BT_FORWARD)
            {
                if (buttons & BT_MOVERIGHT)
                {
                    changepaint = 1;
                    SetFont("PNTS_MAG");
                    TakeInventory("ColorType",0x7FFFFFFF);
                    GiveInventory("ColorType",5);
                }
                else if (buttons & BT_MOVELEFT)
                {
                    changepaint = 1;
                    SetFont("PNTS_RED");
                    TakeInventory("ColorType",0x7FFFFFFF);
                    GiveInventory("ColorType",1);
                }
                else
                {
                    changepaint = 1;
                    SetFont("PNTS_BLK");
                    TakeInventory("ColorType",0x7FFFFFFF);
                }
            }
            else if (buttons & BT_BACK)
            {
                if (buttons & BT_MOVERIGHT)
                {
                    changepaint = 1;
                    SetFont("PNTS_CYA");
                    TakeInventory("ColorType",0x7FFFFFFF);
                    GiveInventory("ColorType",6);
                }
                else if (buttons & BT_MOVELEFT)
                {
                    changepaint = 1;
                    SetFont("PNTS_BLU");
                    TakeInventory("ColorType",0x7FFFFFFF);
                    GiveInventory("ColorType",3);
                }
                else
                {
                    changepaint = 1;
                    SetFont("PNTS_WHT");
                    TakeInventory("ColorType",0x7FFFFFFF);
                    GiveInventory("ColorType",7);
                }
            }
            else if (buttons & BT_MOVERIGHT)
            {
                changepaint = 1;
                SetFont("PNTS_YEL");
                TakeInventory("ColorType",0x7FFFFFFF);
                GiveInventory("ColorType",4);
            }
            else if (buttons & BT_MOVELEFT)
            {
                changepaint = 1;
                SetFont("PNTS_GRN");
                TakeInventory("ColorType",0x7FFFFFFF);
                GiveInventory("ColorType",2);
            }
            else
            {
                changepaint = 0;
                SetFont("PNTS_NOP");
                TakeInventory("ColorType",0x7FFFFFFF);
                GiveInventory("ColorType",prevpaint);
            }

            HudMessage(s:"a";HUDMSG_PLAIN,17,CR_UNTRANSLATED,160.0,100.0,0.2);
            delay(1);
        }
        delay(1);
        SetPlayerProperty(0,0,PROP_FROZEN);
        SetFont("SMALLFONT");
        if (changepaint == 1)
        {
            if (CheckInventory("ColorType") == 0)
            {
                FadeRange(0,0,0,0.25,0,0,0,0,0.5);
                HudMessage(s:"Changed color to BLACK.";HUDMSG_FADEOUT,17,CR_WHITE,160.0,80.0,1.5,1.0);
            }
            if (CheckInventory("ColorType") == 1)
            {
                FadeRange(255,0,0,0.25,0,0,0,0,0.5);
                HudMessage(s:"Changed color to RED.";HUDMSG_FADEOUT,17,CR_WHITE,160.0,80.0,1.5,1.0);
            }
            if (CheckInventory("ColorType") == 2)
            {
                FadeRange(0,255,0,0.25,0,0,0,0,0.5);
                HudMessage(s:"Changed color to GREEN.";HUDMSG_FADEOUT,17,CR_WHITE,160.0,80.0,1.5,1.0);
            }
            if (CheckInventory("ColorType") == 3)
            {
                FadeRange(0,0,255,0.25,0,0,0,0,0.5);
                HudMessage(s:"Changed color to BLUE.";HUDMSG_FADEOUT,17,CR_WHITE,160.0,80.0,1.5,1.0);
            }
            if (CheckInventory("ColorType") == 4)
            {
                FadeRange(255,255,0,0.25,0,0,0,0,0.5);
                HudMessage(s:"Changed color to YELLOW.";HUDMSG_FADEOUT,17,CR_WHITE,160.0,80.0,1.5,1.0);
            }
            if (CheckInventory("ColorType") == 5)
            {
                FadeRange(255,0,255,0.25,0,0,0,0,0.5);
                HudMessage(s:"Changed color to MAGENTA.";HUDMSG_FADEOUT,17,CR_WHITE,160.0,80.0,1.5,1.0);
            }
            if (CheckInventory("ColorType") == 6)
            {
                FadeRange(0,255,255,0.25,0,0,0,0,0.5);
                HudMessage(s:"Changed color to CYAN.";HUDMSG_FADEOUT,17,CR_WHITE,160.0,80.0,1.5,1.0);
            }
            if (CheckInventory("ColorType") == 7)
            {
                FadeRange(255,255,255,0.25,0,0,0,0,0.5);
                HudMessage(s:"Changed color to WHITE.";HUDMSG_FADEOUT,17,CR_WHITE,160.0,80.0,1.5,1.0);
            }
        }
        break;

    case 6:
        fuckthis = unusedTID(2565,32767);
        Thing_ChangeTID(0,fuckthis);
        SetActivator(0,AAPTR_TARGET);
        pln = PlayerNumber();
        Thing_ChangeTID(fuckthis,2000+pln);
        Thing_SetTranslation(2000+pln,-1);
        break;

    case 7:
        if (ActivatorTID() == 0)
        {
            fuckthis = unusedTID(2565,32767);
            Thing_ChangeTID(0,fuckthis);
            SetActivator(0,AAPTR_TARGET);
            pln = PlayerNumber();
            Thing_ChangeTID(fuckthis,2000+pln);
            Thing_SetTranslation(2000+pln,-1);
        }
        break;

    case 8: // ugh this is so brute force but i don't care anymore
        delay(1);
        GiveInventory("MaySwingAgain",1);
        delay(4);
        TakeInventory("KillTheBall",1);
        break;

    case 9:
        SetActivator(0,AAPTR_TARGET);
        pln = PlayerNumber();
        Warp(2500+pln,ballx[pln],bally[pln],ballz[pln],0,WARPF_USECALLERANGLE | WARPF_ABSOLUTEPOSITION | WARPF_NOCHECKPOSITION | WARPF_STOP | WARPF_MOVEPTR);
        break;

    case 10:
        thegallofthisshit = unusedTID(2565,32767);
        Thing_ChangeTID(0,thegallofthisshit);
        SetActivator(0,AAPTR_TARGET);
        pln = PlayerNumber();
        Thing_ChangeTID(thegallofthisshit,2500+pln);
        break;

    case 11:
        if (CheckInventory("CameraOn") == 0)
        {
            LocalAmbientSound("camera/on",127);
            SetPlayerProperty(0,1,PROP_FROZEN); //TOTALLYFROZEN); 
            //ChangeCamera(cameratid,0,0);
            GiveInventory("CameraOn",1);
        }
        else
        {
            //if (CheckInventory("BetweenShots") == 1) { SetPlayerProperty(0,0,PROP_FROZEN); }
            LocalAmbientSound("camera/off",127);
            SetPlayerProperty(0,0,PROP_TOTALLYFROZEN); //TOTALLYFROZEN); 
            //ChangeCamera(0,0,0);
            TakeInventory("CameraOn",1);
        }
        break;

    case 12:
        if (CheckInventory("MaySwingAgain") == 0)
        {
            LocalAmbientSound("ball/cancel",127);
            SetHudSize(320,200,0);
            SetFont("SMALLFONT");
            HudMessage(s:"Shot cancelled.";HUDMSG_FADEOUT,15,CR_GOLD,160.0,85.0,1.5,1.0);
            HudMessage(s:"Alt-fire to try again.";HUDMSG_FADEOUT,16,CR_GOLD,160.0,95.0,1.5,1.0);
            GiveInventory("MaySwingAgain",1);
            GiveInventory("BetweenShots",1);
            Thing_Remove(2000+PlayerNumber());
        }
        break;

    case 13: // renegade's shotgun hack
        x = GetActorX(0);
        y = GetActorY(0);
        z = GetActorZ(0);
        //PrintBold(i:GetActorZ(0) - GetActorFloorZ(0));
        //SetActivatorToTarget(0);
        if (CheckActorFloorTexture(0,"HOLFINIS"))
        {
            //SetActivator(0,AAPTR_MASTER);
            SetActivatorToTarget(0);
            if (CheckInventory("NotPlayingGolf") == 1) { terminate; }

            AmbientSound("hole/in",127);
            if (CheckInventory("StrokeCounter") <= CheckInventory("CourseAceHack")) { LocalAmbientSound("audience/roar",127); }
            else if (CheckInventory("StrokeCounter") <= CheckInventory("CourseParHack")) { LocalAmbientSound("audience/clap",127); }

            SetHudSize(320,200,0);
            SetFont("SMALLFONT");
            HudMessageBold(n:0,s:" finished";HUDMSG_FADEOUT,15,CR_GOLD,160.0,85.0,1.5,1.0);
            if (CheckInventory("StrokeCounter") > 1)
            {
                HudMessageBold(s:"in ",i:CheckInventory("StrokeCounter"),s:" strokes!";HUDMSG_FADEOUT,16,CR_GOLD,160.0,95.0,1.5,1.0);
                HudMessageBold(n:0,s:" finished in ",i:CheckInventory("StrokeCounter"),s:" strokes!";HUDMSG_LOG,0,CR_UNTRANSLATED,0.0,0.0,0.1);
            }
            else
            {
                HudMessageBold(s:"in ",i:CheckInventory("StrokeCounter"),s:" stroke!";HUDMSG_FADEOUT,16,CR_GOLD,160.0,95.0,1.5,1.0);
                HudMessageBold(n:0,s:" finished in ",i:CheckInventory("StrokeCounter"),s:" stroke!";HUDMSG_LOG,0,CR_UNTRANSLATED,0.0,0.0,0.1);
            }

            if (CheckInventory("StrokeCounter") > (CheckInventory("CourseParHack") * 5) && CheckInventory("CourseParHack") != 0)
            {
                quoteshenanigans = random(0,QUOTEAMOUNT - 1);
                HudMessageBold(s:EndingQuotes[quoteshenanigans];HUDMSG_FADEOUT,0,CR_WHITE,160.0,125.0,3.5,1.0);
            }

            GiveInventory("FinishedHole",1);
            GiveInventory("MaySwingAgain",1);
            GiveInventory("BetweenShots",1);
            IFinished[PlayerNumber()] = 1;
            PlayerFinishedNames[PlayersFinished] = Strparam(n:0);
            PlayerFinishedStrokes[PlayersFinished] = CheckInventory("StrokeCounter");
            PlayersFinished++;
            PlayersFinishedScoreboard++;
            if (GameType() == GAME_SINGLE_PLAYER || GetCvar("golf_endgame") == 2)
            {
                delay(35*5);
                Exit_Normal(0);
            }
        }
        else
        {
            if (GetOutOfBounds() == 1)//(CheckActorFloorTexture(0,"OUTBOUND"))
            {
                //GiveInventory("KillMe",1); // Just In Case
                fuckthisshit = unusedTID(2565,32767); // Apparently these landed balls don't have TIDs.
                Thing_ChangeTID(0,fuckthisshit); // So it's temporarily assigned a TID.
                SetActivatorToTarget(0);
                SetActorPosition(fuckthisshit,ballx[PlayerNumber()],bally[PlayerNumber()],ballz[PlayerNumber()],0); // I thought these landed balls have TIDs.
                Thing_ChangeTID(fuckthisshit,0); // Fuck me, right?
                //SetActorPosition(2000+PlayerNumber(),ballx[PlayerNumber()],bally[PlayerNumber()],ballz[PlayerNumber()],0);
                SetActorPosition(2500+PlayerNumber(),ballx[PlayerNumber()],bally[PlayerNumber()],ballz[PlayerNumber()],0); // The cursors do, though, which is dumb.
                if (CheckInventory("NotPlayingGolf") == 1) { terminate; }
                LocalAmbientSound("ball/badland",127);
                SetHudSize(320,200,0);
                SetFont("SMALLFONT");
                HudMessage(s:"Out of bounds!";HUDMSG_FADEOUT,15,CR_GOLD,160.0,85.0,1.5,1.0);
                HudMessage(s:"Alt-fire to try again.";HUDMSG_FADEOUT,16,CR_GOLD,160.0,95.0,1.5,1.0);
                //GiveInventory("KillTheBall",1);
                GiveInventory("MaySwingAgain",1);
                GiveInventory("BetweenShots",1);
                //Thing_Remove(2000+PlayerNumber());
                //ACS_NamedExecuteAlways("GOLF_DECORATE",0,8,0,0);
            }
            else
            {
                SetActorPosition(2500+PlayerNumber(),x,y,z,0); // Move the indicator cursor to this dude's position.
                SetActivatorToTarget(0);
                if (CheckInventory("NotPlayingGolf") == 1) { terminate; }
                ballx[PlayerNumber()] = x;
                bally[PlayerNumber()] = y;
                ballz[PlayerNumber()] = z;
                //SetActorPosition(0,x,y,z,0);
                //LocalAmbientSound("ball/goodland",127);
                //SetHudSize(320,200,0);
                //SetFont("SMALLFONT");
                //HudMessage(s:"Ball nudged!";HUDMSG_FADEOUT,15,CR_GOLD,160.0,85.0,1.5,1.0);
                //HudMessage(s:"Alt-fire to swing.";HUDMSG_FADEOUT,16,CR_GOLD,160.0,95.0,1.5,1.0);
                GiveInventory("BetweenShots",1);
                //ACS_NamedExecuteAlways("GOLF_DECORATE",0,8,0,0);
            }
        }
        break;
    }
}

script "GOLF_CLIENT" (int cocks) CLIENTSIDE
{
    int fuckthis2, pln;

    switch (cocks)
    {
    case 0:
        if (CheckInventory("IsMaestro") == 1) { LocalAmbientSound("maestro/swap",127); }
        else { LocalAmbientSound("club/select",127); }
        break;

    case 1:
        LocalAmbientSound("club/cantshoot",127);
        break;

    case 2:
        if (CheckInventory("IsMaestro") == 1) { LocalAmbientSound("8bpaint/select",127); }
        else { LocalAmbientSound("paint/select",127); }
        break;

    case 3:
        LocalAmbientSound("wand/select",127);
        break;

    case 6:
        fuckthis2 = unusedTID(2565,32767);
        Thing_ChangeTID(0,fuckthis2);
        SetActivator(0,AAPTR_TARGET);
        pln = PlayerNumber();
        Thing_ChangeTID(fuckthis2,2000+pln);
        Thing_SetTranslation(2000+pln,-1);
        break;
    }
}

script "GOLF_RESPAWN" RESPAWN
{
    //Thing_ChangeTID(0,100+PlayerNumber());
    if (GameType() == GAME_NET_DEATHMATCH) { SetActorProperty(0,APROP_SPECIES,"DMPlayer"); }
    else { GiveInventory("DeathsplosionResistance",1); }
    if (ThingCountName("EndFlag",0) == 0) { GiveInventory("NotPlayingGolf",1); }

    if (CheckInventory("IsMaestro") == 1) { ACS_NamedExecuteAlways("GOLF_MAESTRO",0,0,0,0); }
    if (CheckInventory("IsCorzo") == 1) { GiveInventory("ShootableOn",1); }

    // Already covered in GOLF_DEATH, but Just In Case.
    GiveInventory("MaySwingAgain",1);
    GiveInventory("BetweenShots",1);
    GiveInventory("NoClipDeactivate",1);
    TakeInventory("SwingStage1",1);
    TakeInventory("CameraOn",1);
    TakeInventory("SelectingPaint",1);
    TakeInventory("KillTheBall",1);
    TakeInventory("ClubHit",1);
    TakeInventory("ClubSwing",1);
    TakeInventory("ClubSwinging",1);
}

script "GOLF_DISCONNECT" (int pln) DISCONNECT 
{
    int nupln, prevpln;

    Thing_Remove(2000+pln);
    Thing_Remove(1500+pln);
    Thing_Remove(100+pln);
    PlayerStrokes[pln] = 0;

    //if (CheckInventory("FinishedHole") == 1)
    /*if (IFinished[pln] == 1)
    {
        for (prevpln = pln - 1; prevpln < PlayersFinished; prevpln++)//MAX_PLAYERS; ++x)
        {
           nupln = prevpln+1;
           PlayerFinishedNames[prevpln] = PlayerFinishedNames[nupln];
           PlayerFinishedStrokes[prevpln] = PlayerFinishedStrokes[nupln];
        }
        PlayersFinished--;
    }*/

    if (IFinished[pln] == 1) { PlayersFinished--; }
    Thing_ChangeTID(pln,0);
    PlayersInGame--;
}

script "GOLF_RESET" (void)
{
    int pln;
    int buttons, oldbuttons;

    pln = PlayerNumber();

    if (PlayerDeliberating[pln] == 1 || CheckInventory("CameraOn") == 1) { terminate; }
    if (CheckInventory("FinishedHole") == 1) { Print(s:"You've already finished the hole."); terminate; }

    SetActorVelocity(0,0,0,0,0,0);
    SetPlayerProperty(0,1,PROP_TOTALLYFROZEN);
    PlayerDeliberating[pln] = 1;

    LocalAmbientSound("reset/ask",127);

    buttons = GetPlayerInput(-1, INPUT_BUTTONS);
    oldbuttons = GetPlayerInput(-1, INPUT_OLDBUTTONS);
    delay(1);

    while(1)
    {
        buttons = GetPlayerInput(-1, INPUT_BUTTONS);
        oldbuttons = GetPlayerInput(-1, INPUT_OLDBUTTONS);

        SetHudSize(320,200,0);
        SetFont("SMALLFONT");

        if (CheckInventory("FinishedHole") == 1) // Just in case they somehow finish the hole while deliberating whether or not to reset.
        {
            HudMessage(s:" ";HUDMSG_PLAIN,18,CR_GOLD,0.0,0.0,0.1);
            HudMessage(s:" ";HUDMSG_PLAIN,19,CR_GOLD,0.0,0.0,0.1);
            terminate;
        }

        HudMessage(s:"Get a new ball, reset stroke counter?";HUDMSG_FADEOUT,18,CR_WHITE,160.0,70.0,0.5,0.25);
        HudMessage(s:"Use to confirm, reload to cancel.";HUDMSG_FADEOUT,19,CR_WHITE,160.0,110.0,0.5,0.25);

        if (buttons & BT_USE && !(oldbuttons & BT_USE))
        {
            FadeRange(255,255,128,0.33,0,0,0,0,1.25);

            PlayerStrokes[pln] = 0;
            TakeInventory("StrokeCounter",0x7FFFFFFF);
            ballx[pln] = startballx[pln];
            bally[pln] = startbally[pln];
            ballz[pln] = startballz[pln];

            GiveInventory("MaySwingAgain",1);
            GiveInventory("BetweenShots",1);
            GiveInventory("GolfHealthRestore",100);

            Thing_Remove(2000+PlayerNumber());
            Thing_Remove(2500+PlayerNumber());

            LocalAmbientSound("reset/confirm",127);
            break;
        }
        if (buttons & BT_RELOAD && !(oldbuttons & BT_RELOAD))
        {
            HudMessage(s:"Get a new ball, reset stroke counter?";HUDMSG_FADEOUT,18,CR_WHITE,160.0,70.0,0.01,0.25);
            HudMessage(s:"Use to confirm, reload to cancel.";HUDMSG_FADEOUT,19,CR_WHITE,160.0,110.0,0.01,0.25);
            LocalAmbientSound("reset/cancel",127);
            break;
        }

        delay(1);
    }

    delay(1);
    SetPlayerProperty(0,0,PROP_TOTALLYFROZEN);
    PlayerDeliberating[pln] = 0;
}

script "GOLF_HELLSHOT" (void)
{
    SetHudSize(640,400,0);
    SetFont("SMALLFONT");

    if (CheckInventory("PoweredUp") == 0)
    {
        if (CheckInventory("HellshotStock") >= 1)
        {
            FadeRange(190,96,0,0.75,0,0,0,0,1.0);
            GiveInventory("PoweredUp",1);
            TakeInventory("HellshotStock",1);
            TakeInventory("HellshotAmmo",1750);
            LocalAmbientSound("power/up",127);

            if (CheckInventory("IsDoomguy") == 1) { GiveInventory("PoweredDoomguy",1); }
            if (CheckInventory("IsMaestro") == 1) { GiveInventory("PoweredMaestro",1); }
            if (CheckInventory("IsCygnis") == 1) { GiveInventory("PoweredCygnis",1); }
            if (CheckInventory("IsCKid") == 1) { GiveInventory("PoweredCKid",1); }
            if (CheckInventory("IsRekkr") == 1) { GiveInventory("PoweredRekkr",1); }
            if (CheckInventory("IsThi") == 1) { GiveInventory("PoweredThi",1); }
            if (CheckInventory("IsLordBlaz") == 1) { GiveInventory("PoweredLordBlaz",1); }
            if (CheckInventory("IsRenegade") == 1) { GiveInventory("PoweredRenegade",1); }
            if (CheckInventory("IsDaina") == 1) { GiveInventory("PoweredDaina",1); }
            if (CheckInventory("IsVesper") == 1) { GiveInventory("PoweredVesper",1); }
            if (CheckInventory("IsCorzo") == 1) { GiveInventory("PoweredCorzo",1); }
            if (CheckInventory("IsSquare") == 1) { GiveInventory("PoweredSquare",1); }
            if (CheckInventory("IsMsSkeltal") == 1) { GiveInventory("PoweredMsSkeltal",1); PrintBold(s:"thank ms skeltal"); }

        }
        else
        {
            GiveInventory("HellshotNope",1);
            LocalAmbientSound("power/nope",127);
            HudMessage(s:"You don't have enough";HUDMSG_FADEOUT,22,CR_GOLD,320.0,350.0,1.5,1.0);
            HudMessage(s:"power to perform a Hellshot.";HUDMSG_FADEOUT,23,CR_GOLD,320.0,360.0,1.5,1.0);
            delay(35);
            TakeInventory("HellshotNope",1);
        }
    }
    else
    {
        FadeRange(0,0,0,0.75,0,0,0,0,0.666); // omgsatan
        TakeInventory("PoweredUp",1);
        LocalAmbientSound("power/down",127);
        GiveInventory("HellshotAmmo",1750);
        GiveInventory("HellshotStock",1);
    }
}